/*
 * $Id: $
 *
 * Copyright (c) 2006 Network Appliance, Inc.
 * All rights reserved.
 */

package netapp.manage;

import netapp.manage.Base16;

/**
 * A utility class implementing ARCFOUR (the alleged RC4 algorithm)
 * for encryption of a data stream. Until 1994 RC4 was a trade
 * secret of RSA Data Security, Inc, when it was released anonymously
 * to a mailing list.  Because there is no proof that the leaked
 * version was in fact RC4 and because "RC4" is a trademark, it is
 * called "ARCFOUR", short for "Allegedly RC4".
 *
 * This version is ported to java from the source in zephyr's
 * src/libnetapp/na.c.
 * 
 * This algorithm has the property that calling crypt() with the
 * same password on an encrypted value returns the original text.
 */
public class ARCFour {

    /*
     * The algorithm specifies an "S" box of this size.
     */
    private static final int STATE_SIZE = 256;
    
    /*
     * If a caller doesn't specify a key, use this one.  This
     * value matches the one in libnetapp/na.c.
     */
    private static final String DEFAULT_KEY = "#u82fyi8S5\017pPemw";

    /*
     * The key we're using for encryption.
     */
    private String key;

    /*
     * These fields are part of the "context" for an encryption operation.
     */
    private char state[] = new char[STATE_SIZE];
    private int x;
    private int y;

    /**
     * Constructor using a default key.
     */
	public ARCFour() {
		this(DEFAULT_KEY);
	}
    
    /**
     * Constructor using a key you specify.
     */
    public ARCFour(String key) {
        this.key = key;
    }

    /**
     * Initializes the crypto engine.  Call this method before
     * each unique encryption or decryption operation.
     */
    public void init() {
        x = 0;
        y = 0;
        for (char i = 0; i < STATE_SIZE; i++) {
            state[i] = i;
        }
        int j = 0;
        int k = 0;

        for (int i = 0; i < STATE_SIZE; i++) {
            k = (key.charAt(j) + state[i] + k) % STATE_SIZE;
            swap(i, k);
            j = (j + 1) % key.length();
        }
    }
    
    /**
     * Swaps two state bytes (identified by index).
     */
    private void swap(int i, int j) {
        char swapByte;

        swapByte = state[i];
        state[i] = state[j];
        state[j] = swapByte;
    }
    
    /**
     * Returns the next byte to use for XOR-ing with the source data.
     * This also advances the internal x and y indices.
     */
    private char nextByte() {
        int newX = (this.x + 1) % STATE_SIZE;
        int newY = (state[newX] + this.y) % STATE_SIZE;

        swap(newX, newY);
        
        this.x = newX;
        this.y = newY;
        
        int j = (state[newX] + state[newY]) % STATE_SIZE;
        return state[j];
    }

    /**
     * Encrypts or decrypt.  This can be called multiple times
     * to encrypt a stream of data.  Be sure to call init() before
     * each unique encryption or decryption operation.
     * 
     * @param input characters to encrypt; this may be null
     * @param output location to store the encrypted result
     * @return the encrypted result
     */
    public char[] crypt(char[] input, char[] output) {
        if (null == input) {
            return null;
        }
        for(int i = 0; i < input.length; i++) {
            output[i] = (char) (input[i] ^ nextByte());
        }
        return output;
    }
    
    /**
     * Encrypts or decrypt an input buffer.
     * 
     * @param input characters to encrypt; this may be null
     * @return the encrypted result
     */
    public char[] crypt(char[] input) {
        if (null == input) {
            return null;
        }
        char output[] = new char[input.length];
        return crypt(input, output);
    }
    
    /**
     * Encrypts or decrypt a string.
     * 
     * @param input string to encrypt; this may be null
     * @return the encrypted result
     */
    public char[] crypt(String input) {
        if (null == input) {
            return null;
        }
        return crypt(input.toCharArray());
    }
    
    /**
     * Convenience method that both encrypts and base-16 encodes
     * a string.  This is suitable for use in ZAPI calls.
     *
     * We don't expose this flavor for now, because we have no
     * use (in ZAPI) for anything other than the default key, and
     * don't want to tempt developers into using non-default keys.

     */
    static String encryptAndEncode(String input, String key) {
        ARCFour engine = new ARCFour(key);
        engine.init();
        return Base16.encode(engine.crypt(input));
    }
    
    /**
     * Convenience method that both encrypts and base-16 encodes
     * a string using the default key.
     *
     * @param input plaintext to encrypt; this may be null
     * @return the encrypted, encoded string
     */
    public static String encryptAndEncode(String input) {
        return encryptAndEncode(input, DEFAULT_KEY);
    }
    
    /**
     * Convenience method that undoes the encoding and encryption from
     * encryptAndEncode().
     *
     * We don't expose this flavor for now, because we have no
     * use (in ZAPI) for anything other than the default key, and
     * don't want to tempt developers into using non-default keys.
     */
    static String decodeAndDecrypt(String input, String key) {
        if (null == input) {
            return null;
        }
        ARCFour engine = new ARCFour(key);   
        engine.init();
        return new String(engine.crypt(Base16.decode(input)));
    }
    
    /**
     * Convenience method that undoes the encoding and encrypt from
     * encryptAndEncode() using the default key.
     * 
     * @param input encrypted/encoded data to decode; this may be null
     * @return the decoded, decrypted (plaintext) string
     */
    public static String decodeAndDecrypt(String input) {
        return decodeAndDecrypt(input, DEFAULT_KEY);
    }
}
