/*
 * $Id: //depot/prod/zephyr/lufthansaair/src/java/netapp/manage/NaParser.java#1 $
 *
 * Copyright (c) 2004 Network Appliance, Inc.
 * All rights reserved.
 *
 * ONTAPI Server stuff
 */

package netapp.manage;

import java.io.Reader;
import java.io.IOException;
import java.util.Stack;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 *  This class is needed to parse the XML responses coming back
 *  from API invocations. It's not exposed to SDK users.
 */
final class NaParser {

	private static final boolean DO_SYSLOG = false;
	private static final boolean VALIDATE = false;

	private static SAXParserFactory _factory;

	static
	{
		_factory = SAXParserFactory.newInstance();
		_factory.setValidating(VALIDATE);

		/*
		 *  This namespace stuff doesn't seem to be
		 *  doing anything.
		 */
		_factory.setNamespaceAware(true);
	}

	/*
	 *  Prevent instantiation of this class
	 */
	private NaParser()
	{
		;
	}

	static NaElement parseTree(Reader in) throws IOException, SAXException
	{
		XMLReader xr;

		try {
			xr = _factory.newSAXParser().getXMLReader();
		} catch (ParserConfigurationException e) {
			throw new NaError("Failed to get XML parser", e);
		}

		/*
		 *  XXX - what on earth do these do?
		 */
		xr.setFeature("http://xml.org/sax/features/validation", VALIDATE);
		xr.setFeature("http://xml.org/sax/features/namespaces", true);
		xr.setFeature("http://xml.org/sax/features/namespace-prefixes",
			true);

		if (DO_SYSLOG) {
			/*
			 *  Set up and error logging object
			 */
			xr.setErrorHandler(new ErrorHandler() {
				public void warning(SAXParseException e)
					throws SAXParseException
				{
					logError(e);
					throw e;
				}

				public void error(SAXParseException e)
					throws SAXParseException
				{
					logError(e);
					throw e;
				}

				public void fatalError(SAXParseException e)
					throws SAXParseException
				{
					logError(e);
					throw e;
				}

				private void logError(SAXParseException e)
				{
					System.err.println("XML parsing error on line " +
						e.getLineNumber() + ": " + e.getMessage());
				}
			});
		}

		/*
		 *  Create an instance of our handler object, which
		 *  handles the callbacks from the SAX parser.
		 */
		NaHandler h = new NaHandler();
		xr.setContentHandler(h);

		/*
		 *  Parse the document
		 */
		xr.parse(new InputSource(in));

		/*
		 *  Return the root element of the tree
		 */
		return h._rootElem;
	}

	/*
	 *  This class listens to SAX parser callbacks and creates
	 *  a tree of NaElemen objects.
	 */
	private static class NaHandler extends DefaultHandler {

		private StringBuffer _currentToken;
		private Stack _tokens;
		private Stack _elementStack;
		private NaElement _currentElem;

		NaElement _rootElem;

		NaHandler()
		{
			_currentToken = new StringBuffer();
			_tokens = new Stack();
			_elementStack = new Stack();
			_currentElem = null;
			_rootElem = null;
		}

		/*
		 *  SAX callbacks
		 */

		public void startElement(String uri, String name, String qname,
			Attributes attrs)
		{
			/*
			 *  Save the current token
			 */
			_tokens.push(_currentToken);
			_currentToken = new StringBuffer();

			/*
			 *  Also save the current element
			 */
			_elementStack.push(_currentElem);
			_currentElem = new NaElement(qname, null, attrs);
		}

		public void endElement(String uri, String name, String qname)
		{
			/*
			 *  Stick the current token into the current element's
			 *  content.
			 */
			if (0 < _currentToken.length()) {
				_currentElem.setContent(_currentToken.toString());
			}
			_currentToken = (StringBuffer) _tokens.pop();

			/*
			 *  Make the previous element the current one, after
			 *  adding the child
			 */
			NaElement popped = (NaElement) _elementStack.pop();
			if (null == popped) {
				_rootElem = _currentElem;
			} else {
				popped.addChildElem(_currentElem);
			}
			_currentElem = popped;
		}

		public void characters(char[] buf, int offset, int len)
		{
			_currentToken.append(buf, offset, len);
		}
	}
}
