/*
 * $Id: //depot/prod/zephyr/lufthansaair/src/java/netapp/manage/http/HTTPRequest.java#1 $
 *
 * Copyright (c) 2001-2004 Network Appliance, Inc.
 * All rights reserved.
 *
 * ONTAPI Server stuff
 */

package netapp.manage.http;

import java.io.InputStream;
import java.io.IOException;
import java.util.StringTokenizer;

/**
 *  This class represents a HTTP request.
 */
public class HTTPRequest extends HTTPMessage {

	private int _method;
	private String _path;

	/**
	 *  Constructs a <code>HTTPRequest</code>.
	 *
	 *  @param	method		the HTTP request method.
	 *  @param	path		the URI path to request.
	 *  @param	version		the HTTP version.
	 */
	public HTTPRequest(int method, String path, int version)
	{
		super(version);
		_method = method;
		_path = path;
	}

	/**
	 *  Get the HTTP method of this request. It will be either
	 *  {@link netapp.manage.http.HTTPMessage#METHOD_GET} or
	 *  {@link netapp.manage.http.HTTPMessage#METHOD_POST}.
	 *
	 *  @return	the HTTP method.
	 */
	public int getMethod()
	{
		return _method;
	}

	/**
	 *  Get the URI-path for this HTTP request.
	 *
	 *  @return	a URI-path string.
	 */
	public String getPath()
	{
		return _path;
	}

	/**
	 *  Add a header for HTTP Basic authentication.
	 *
	 *  @param	username	the username.
	 *  @param	password	the password.
	 */
	public void setBasicAuthorization(String username, String password)
	{
        String credentialString = username + ":" + password;

        byte[] credentialBytes = null;

        try {
            credentialBytes = credentialString.getBytes("UTF-8");
        } catch(java.io.UnsupportedEncodingException e) {
            credentialBytes = credentialString.getBytes();
        }

		setHeader("Authorization", "Basic " +
			Base64.encodeBytes(credentialBytes, false));
	}

	/**
	 *  Create a new HTTP request by reading data from a socket.
	 *
	 *  @param	in	an <code>InputStream</code>, presumably
	 *			connected to a server socket.
	 *  @return		a new <code>HTTPRequest</code>.
	 *  @exception	IOException	if there is a communication error on
	 *				the <code>InputStream</code>.
	 *  @exception	HTTPException	if the HTTP request is malformed.
	 */
	public static HTTPRequest read(InputStream in)
		throws IOException, HTTPException
	{
		String arguments = readLine(in);

		StringTokenizer st = new StringTokenizer(arguments);
		if (st.countTokens() != 3) {
			throw new HTTPException("Malformed request: " + arguments);
		}

		int method = methodFromString(st.nextToken());
		String path = st.nextToken();
		int version = versionFromString(st.nextToken());

		HTTPRequest request = new HTTPRequest(method, path, version);

		for (String line = readLine(in); 0 < line.length();
				line = readLine(in)) {
			request.processHeaderLine(line);
		}

		if (METHOD_GET == method) {
			return request;
		}

		request.readContents(in);

		return request;
	}

	/**
	 *  Get the first line of this HTTP request.
	 *
	 *  @return	a string representing one line.
	 */
	protected String firstLine()
	{
		return methodToString(_method) + " " + (_path + " " +
			versionToString(_version)).trim() + "\r\n";
	}
}
